@@ -1,7 +1,7 @@
 package ExclusiveLock::Guard;
 use strict;
 use warnings;
-our $VERSION = '0.04';
+our $VERSION = '0.07';
 
 use Errno qw(EWOULDBLOCK);
 use Fcntl qw(LOCK_EX LOCK_NB LOCK_UN);
@@ -72,14 +72,27 @@ sub DESTROY {
 
     my $fh       = delete $self->{fh};
     my $filename = delete $self->{filename};
-    unless (unlink $filename) {
-        warn "failed to unlink file:$filename:$!";
-    }
-    unless (flock $fh, LOCK_UN) {
-        warn "failed to unlock flock file:$filename:$!";
-    }
     unless (close $fh) {
         warn "failed to close file:$filename:$!";
+        return;
+    }
+
+    # try unlink lock file
+    if (open my $unlink_fh, '<', $filename) { # else is unlinked lock file by another process?
+        # A
+        if (flock $unlink_fh, LOCK_EX | LOCK_NB) { # else is locked the file by another process
+            if (-f $filename && stat($unlink_fh)->ino == do { my $s = stat($filename); $s ? $s->ino : -1 }) { # else is unlink and create file by another process in the A timing
+                unless (unlink $filename) {
+                    warn "failed to unlink file:$filename:$!";
+                }
+                unless (flock $unlink_fh, LOCK_UN) {
+                    warn "failed to unlock flock file:$filename:$!";
+                }
+                unless (close $unlink_fh) {
+                    warn "failed to close file:$filename:$!";
+                }
+            }
+        }
     }
 }
 
@@ -126,6 +139,10 @@ To automatically create and remove the lock file.
 
 Kazuhiro Osawa E<lt>yappo {at} shibuya {dot} plE<gt>
 
+=head1 COPYRIGHT
+
+Copyright 2012- Kazuhiro Osawa
+
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify
